#
#   For_cHX_Step3f.R
#       /DS/Misc/Seurat_To_cellHarmony/May2020/

#   May 10, 2020
#
#   This code was written with Seurat version 3.1.3
#   on a MacBook running R 3.6.1
#
#   Function to create Clustering-exp file
#   from flat files derived from seurat integrated analysis
#
#   Developed to be called by Seurat3Int_To_cHX_Step3.R

# ** User should not need to make changes to this function

For_cHX_Step3f=function(exp_file=NULL,
                       groups_file=NULL,
                       markers_file=NULL,
                       MaxMarkers=50,
                       MoreThan1Cluster="only1",
                       OutputPath=NULL)
{
  # exp_file=exp_file
  # groups_file=groups_file
  # markers_file=markers_file
  # MaxMarkers=50
  # MoreThan1Cluster="only1"
  # OutputPath=outDataPath
  
  
#  Permitted values for input parameters   
  MoreThan1Cluster.vals=c("ok","only1") 

#  CHECKS OF VALUES PROVIDED FOR MaxMarkers and MoreThan1Cluster
  if (!is.numeric(MaxMarkers) | MaxMarkers < 1)
   { stop("Please specify a positive integer for MaxMarkers") }   
  if (!(MoreThan1Cluster %in% MoreThan1Cluster.vals))
   { stop(paste("Allowed values for MoreThan1Cluster are: ",paste(MoreThan1Cluster.vals,collapse = ", "),sep="")) }  

# = = = BEGIN PROCESSING OF MarkerGeneFile = = = = 
message(" ** Begin MarkerGeneFile processing **")

#  Limit to MaxMarkers in each cluster and
#  remove any marker repeats if MoreThan1Cluster=="only1"
clusterList=unique(as.character(markers_file$cluster))
nclusters=length(clusterList)
MarkersAlreadyUsed=c(NULL)
mfile=markers_file

for (indx in 1:nclusters)
{
#indx=1
#   extract all marker info for the cluster
  a=mfile[mfile$cluster==clusterList[indx],]
  
#   remove already used markers if requested
  if (MoreThan1Cluster=="only1") {
    a=a[!(a$gene %in% MarkersAlreadyUsed),]
  }  
#   take top MaxMarkers, or all if not that many
  mm=min(MaxMarkers,nrow(a))
  a=a[1:mm,]
  
# store selected markers in genesFile  
  if (indx==1) {genesFile=a}
  if (indx >1) {genesFile=rbind.data.frame(genesFile,a,stringsAsFactors = F)}
  
  #  THE FOLLOWING LINE REMOVES the just processed cluster data from mfile, 
  #  leaving next cluster at top of mfile
  mfile=mfile[! (mfile$cluster %in% clusterList[indx]),]
  
#  add the marker genes to MarkersAlreadyUsed
  MarkersAlreadyUsed=c(MarkersAlreadyUsed,a$gene)
  
}

genesFile=data.frame(indx=1:nrow(genesFile),genesFile,
                     stringsAsFactors = F)

#  checks 
# table(genesFile$cluster,useNA = "always")
# sum(duplicated(genesFile$gene))
# head(genesFile)


marker.df=genesFile
rm(genesFile,mfile,a)
marker.genes=marker.df$gene
message(" ** Finished MarkerGeneFile processing **")


# = =  CREATE THE Clustering-exp. FILE FOR REF SAMPLE = =  
#  example entries are log-normalized
#  *** FORMAT the Clustering-exp. file is
#  UID	               row_clusters-flat	TGGATAGGGGTCn	GAGCCTTCGTAGn
# column_clusters-flat	 	[blank]             0	                 0
# Col1a2	                  0	                4.630085629	4.830100959
# Col1a1	                  0	                0	          4.428620758
# Meg3	                    0                	0                	0
#  *****************************************************************

#  * * * Some cell names might not be legal R names,
#  i.e. for use as rownames or colnames of a matrix or df.
#  To handle this, assign dummy cell names and assign
#  real ones when writing out the file.
message(" ** Begin creating Clustering-exp file **")

cellnames=colnames(exp_file)[2:ncol(exp_file)]
cellnameConv=data.frame(Orig=cellnames,
                        Orig.order=1:length(cellnames),
                        stringsAsFactors = F)
cellnameConv$Dummy=paste("cell",cellnameConv$Orig.order,sep="")

names(exp_file)[2:ncol(exp_file)]=cellnameConv$Dummy

#  Make columns 1 and 2 of the final file
UID=c("column_clusters-flat",marker.df$gene)   #  column 1
r_c_f=c(" ",as.character(marker.df$cluster))  #  change name to row_clusters-flat later

#  Create a df that has data for each marker gene, 
#  allowing for possible repeatedly listed genes (rows)
#  extract expression for marker gene list
rownames(exp_file)=exp_file$UID
exp_file[1:5,1:5] # a check
SubDF=data.frame(exp_file[marker.df$gene,-1],row.names = 1:nrow(marker.df))
# SubDF[1:5,1:5] # a check

#  get cluster info for each cell 
cellClusters=data.frame(cell.name=groups_file$cell_id,
  cluster=groups_file$cluster_number,
  stringsAsFactors = F)
cellClusters$cluster=as.character(cellClusters$cluster)

cluster.cnts=as.vector(table(cellClusters$cluster))
#  add in dummy cell name
cellClusters=merge(x=cellClusters,y=cellnameConv,
                   by.x="cell.name",by.y="Orig",
                   all=T)

#  must order the cells in the cellCluster file by cluster
#  make a factor to reproduce sorting as in groups file
cellClusters$clusterF=factor(cellClusters$cluster,
                             levels=clusterList,ordered = T)
cellClusters=cellClusters[order(cellClusters$clusterF,cellClusters$Orig.order),]

#  order the columns (cells) in the expression matrix same as in the cellClusters df
SubDF=SubDF[,cellClusters$Dummy]
# SubDF[1:5,1:5] # a check

#  create the column_clusters vector for the cells
#  just use cellClusters$cluster since since cells are now ordered the same way in SubDF and cellClusters df
column_clusters=cellClusters$cluster
# column_clusters[1:50]  #  a check

#  make cluster num the 1st row of data DF
SubDF=rbind.data.frame(column_clusters,SubDF,stringsAsFactors = F)

#   put all info together
df=cbind.data.frame(UID,r_c_f,SubDF,stringsAsFactors=F)
df[1,1]="column_clusters-flat"
names(df)[2]="row_clusters-flat"
df[1:8,1:8]   

# checks.  results should be 0
# nrow(df) - (nrow(marker.df) + 1)
# ncol(df) - (ncol(exp_file) + 1)

#  Create a vector with properly sorted cell.name
#  for writing out data
headr=c(colnames(df)[1:2])
headr=c(headr,cellClusters$cell.name)

#    write out Clustering-exp file
f=paste(outDataPath,"Clustering-exp.cHX.txt",sep=.Platform$file.sep)
message(" ** Begin writing ",f)
write.table(df,file=f,quote=F,sep="\t",na = " ",
            row.names = F,col.names = headr)  
message(f," written")

##### END OF For_cHX_Step3f function.  No return value

}


