#

#   Seurat3Int_To_cHX_Step1.R
#   /DS/NF/Seurat3IntVignette
#
#    
#   May 10, 2020
#   Sept 15, 2020 to update SeuratData package usage
#
#   This code was written with Seurat version 3.1.3
#   on a MacBook running R 3.6.1
#
#  cHX Step1
#  This program follows the seurat3 integration vignette,
#  but with only the two celseq datasets and
#  a reduced number of clusters.
#  https://satijalab.org/seurat/v3.1/integration.html
#
#  The output is an integrated seurat object 
#  and a clusterMarkers file (tab-delim)
#  that is the output from the seurat FindConservedMarkers
#  function, with an added variable named "gene".

# ** ** ** ** IMPORTANT ** ** ** ** **
# Valid R names include only letters, numbers, "." ,"_". 
# These restrictions often cause problems when
# cell names/barcodes are used as data.frame column names!!!
# It is advisable to eliminate these from 
# cell names BEFORE beginning this workflow!!

#  This program for Step 1 does not change
#  naming-- it is done, if needed, in the Step 2 program
# ** ** ** ** ** ** ** ** ** ** ** **

#  = = = = = = = =  Setup = = = = = = = = 

R.version.string
library(Seurat)
packageVersion("Seurat")

#  = = USE SeuratData PACKAGE TO GET DATASETS = = 
#devtools::install_github('satijalab/seurat-data')
#  See github site for more info on this package

library(SeuratData)
# InstallData("panc8")
data("panc8")

other.packs=c("Matrix","ggplot2")
lapply(other.packs,library, character.only = TRUE) 

in1DataPath="/DS/NF/Seurat3IntVignette/"
outDataPath=in1DataPath
setwd(in1DataPath)

# ****  1.  FOLLOW SEURAT INTEGRATION VIGNETTE 

pancreas.list <- SplitObject(panc8, split.by = "tech")

#  Just use the 2 cellseqs for this demo
pancreas.list <- pancreas.list[c("celseq", "celseq2")]
#  Note that for some reason the RNA$counts slot
#  has some non-integer values--not sure why.

#  Log-normalize and find HVGs
for (i in 1:length(pancreas.list)) {
  pancreas.list[[i]] <- NormalizeData(pancreas.list[[i]], verbose = FALSE)
  pancreas.list[[i]] <- FindVariableFeatures(pancreas.list[[i]], selection.method = "vst", 
                                             nfeatures = 2000, verbose = FALSE)
}

pancreas.anchors <- FindIntegrationAnchors(object.list = pancreas.list, dims = 1:30)

pancreas.integrated <- IntegrateData(anchorset = pancreas.anchors, dims = 1:30)

# switch to integrated assay. The variable features of this assay are automatically
# set during IntegrateData
DefaultAssay(pancreas.integrated) <- "integrated"

# Run the standard workflow for visualization and clustering
pancreas.integrated <- ScaleData(pancreas.integrated, verbose = T)
pancreas.integrated <- RunPCA(pancreas.integrated, npcs = 30, verbose = FALSE)
#ElbowPlot(pancreas.integrated)
pancreas.integrated <- RunUMAP(pancreas.integrated, reduction = "pca", dims = 1:30)
# p1 <- DimPlot(pancreas.integrated, reduction = "umap", group.by = "tech",
#               size=0.01)
# p2 <- DimPlot(pancreas.integrated, reduction = "umap", group.by = "celltype", label = TRUE, 
#               repel = TRUE) + NoLegend()
#p1
#p2

#  Look at previously-determined clusters
addmargins(table(pancreas.integrated@meta.data$celltype,useNA = "always"))
addmargins(table(pancreas.integrated@meta.data$celltype,
      pancreas.integrated@meta.data$dataset))

#  To avoid small n problems in FindConservedMarkers, 
# subset to biggest clusters with good representation in both samples
clusters.keep=c("acinar","alpha","beta","delta",
                "ductal")

integ.so=SubsetData(pancreas.integrated,subset.name = "celltype",
                   accept.value = clusters.keep)

#  Create a numeric cluster variable,
#  start w/ 0 to mimic seurat convention
integ.so@meta.data$clustNum=NA
for (cl in 1:length(clusters.keep)) {
  integ.so@meta.data$clustNum=ifelse(integ.so@meta.data$celltype==clusters.keep[cl],
                                    cl-1,integ.so@meta.data$clustNum)
}
addmargins(table(integ.so@meta.data$celltype,integ.so@meta.data$clustNum))

#  Find markers for the previously determined clusters
clusters=unique(integ.so@meta.data$clustNum)
clusters=clusters[order(clusters)]

all_markers=data.frame(stringsAsFactors = F)
DefaultAssay(integ.so)="RNA"
Idents(integ.so) = integ.so@meta.data$clustNum
addmargins(table(Idents(integ.so)))

for (cl in clusters) {
#  be a little strict on markers to shorten run time
  markers=FindConservedMarkers(integ.so,ident.1 = cl,
                               grouping.var="dataset",only.pos=T,min.pct = 0.05,
                               min.diff.pct = 0.05,return.thresh = 0.05,
                               logfc.threshold = log(1.5))
  markers$cluster=cl
# Because R rownames can be fussy about duplicates and certain characters,
# create the gene variable to hold the feature names
  markers$gene=rownames(markers)
  all_markers=rbind.data.frame(all_markers,markers,stringsAsFactors = F)
}

table(all_markers$cluster)
#  > 50 for each cluster

#  Might choose to remove dup markers here
#  so don't need to do it later
#dup.markers=duplicated(all_markers$gene)
#sum(dup.markers)   # 41
#all_markers=all_markers[!(dup.markers),]
#table(all_markers$cluster)
#  still > 50 for each cluster

#  = = = = WRITE OUT THE SEURAT OBJECT AND MARKER FILE = = 

f=paste(outDataPath,"integ_so_FromStep1.Rds",sep="")
message(" ** Writing out ",f," **",sep="")
saveRDS(integ.so,file=f)

f=paste(outDataPath,"integ_so_clusterMarkers.txt",sep="")
message(" ** Writing out ",f," **",sep="")
write.table(all_markers,file=f,sep="\t",col.names = T,
            row.names = F,quote=F)

# ** Reminder: If cell names have special
#  characters (e.g.,"-","/") or spaces,
#  they may be changed in the Step 2 program

#######  END OF PROGRAM #############

quit()
