#

#   Seurat3Int_To_cHX_Step2.R
#   /DS/NF/Seurat3IntVignette
#
#    
#   May 10, 2020
#
#   This code was written with Seurat version 3.1.3
#   on a MacBook running R 3.6.1
#
#  Step 2 in the Seurat3Int_To_cHX workflow 
#  Pulls information from the integrated
#  seurat object to prepare files and
#  write out expression & groups files for cHX
# 
# ** ** ** ** IMPORTANT ** ** ** ** **
# Valid R names include only letters, numbers, "." and "_".
# These restrictions often cause problems when
# cell names/barcodes are used as data.frame column names!!!
# !! This program replaces illegal R colname characters 
# with "." (default R behavior) in cell names in 
# the column names of the expression file
#  and the cell_id column of the groups file !!

#  ** Code for user to adjust is preceded by # ////

#  = = = = = = = =  Setup = = = = = = = = 

R.version.string
library(Seurat)
packageVersion("Seurat")

other.packs=c("Matrix")
lapply(other.packs,library, character.only = TRUE) 

# //// Provide paths for input and output files
in1DataPath="/DS/NF/Seurat3IntVignette/"
outDataPath=in1DataPath
setwd(in1DataPath)

# = = =  Bring in integrated seurat object and marker info = = 
# //// Change input filenames if needed
f=paste(in1DataPath,"integ_so_FromStep1.Rds",sep="")
message(" ** Reading ",f," **",sep="")
so=readRDS(f)

f=paste(in1DataPath,"integ_so_clusterMarkers.txt",sep="")
message(" ** Reading ",f," **",sep="")
markerInfo=read.table(file=f,sep="\t",header=T,stringsAsFactors=F)


# ////  For downstream work, need the following 2 columns:
#  gene = the name of the feature
#  cluster = a numeric variable for cluster designation
#  If need to, create variables with those names
markerInfo$gene=markerInfo$gene
# ** be careful if cluster numbering taken from a character variable
markerInfo$cluster=markerInfo$cluster

# = = = = CREATE THE EXPRESSION MATRIX = = = = = 
message(" ** Creating expression matrix **")

# Keep original names to check if R changes some
orig_cellnames=so@assays$RNA@counts@Dimnames[[2]]

# Go back to the RNA counts slot so get all the features (genes)
#  (data slots probably have loge data and a reduced set of features.) 
Cnts.mat=as.matrix(so@assays$RNA@counts)

# Put expression data on usual cH/AA log2(1+CPTT) scale
log2CPTT.mat=log2(1 + 10000 * t(t(Cnts.mat)/colSums(Cnts.mat)))
#rm(Cnts.mat)

exp.df=data.frame(UID=rownames(log2CPTT.mat),log2CPTT.mat,
                       stringsAsFactors = F)

# check for dupped cell identifiers that may cause problems
sum(duplicated(colnames(exp.df)))

# determine if any cell names have been changed by R
cellnames.same=identical(colnames(exp.df)[2:ncol(exp.df)],
                             orig_cellnames)

if (!cellnames.same){
  message(" ** Some cell names were changed to conform to R names")}

#  = = = = Prepare the groups file = = = = = =
message(" ** Preparing the groups file **")

#  Extract cell--cluster info from meta slot
meta=so@meta.data

#  Update cell names if they were changed
if (!cellnames.same){ rownames(meta)=colnames(exp.df)[2:ncol(exp.df)] }


#  Required variables in groups file are:
#  cell_id,  cluster_number, dataset_name,  dataset_type
# ////  Specify variables for cluster_number & dataset_name
grps.file=data.frame(cell_id=rownames(meta),
                     cluster_number=meta$clustNum,
                     dataset_name=meta$tech,
                     stringsAsFactors = F)

# //// Specify variable for Ref / Query designation
grps.file$dataset_type=ifelse(grps.file$dataset_name=="celseq","R","Q")

# ////  cluster_name is an optional variable
grps.file$cluster_name=meta$celltype

# = = = WRITE OUT THE EXPRESSION & GROUPS FILES = = = 

f=paste(outDataPath,"exp.cHX.txt",sep="")
message(" ** Writing ",f," ** ")
write.table(exp.df,file=f,sep="\t",
            col.names = T,row.names = F,quote = F)


f=paste(outDataPath,"groups.cHX.txt",sep="")
message(" ** Writing ",f," ** ")
write.table(grps.file,file=f,sep="\t",
            col.names = T,row.names = F,quote = F)

#######   END OF PROGRAM ###########

1
2
3

quit()
